#include "stdafx.h"
#include <time.h>

//char * g_szConfigSessionTitle = "QUBIO Session Configuration";

class CVwConfigSession : public CDialog { 
	public : 
		CConfigurations m_oCfgs;
		CSession * m_pSession;

		//-- Encapsulate dialog controls 		
		CDlgCombo m_cmbHdwConfig;

		CDlgEdit m_edtGenBufSize;
		CDlgEdit m_edtGenRate;
		CDlgControl m_alblGenChnName[SESSION_MAXGENCHN];
		CDlgControl m_abtnGenBrowse[SESSION_MAXGENCHN];
		CDlgEdit m_aedtGenFileName[SESSION_MAXGENCHN];
		CDlgCheck m_achkGenRepeat[SESSION_MAXGENCHN];
		
		CDlgControl m_btnDigGenBrowse;
		CDlgEdit m_edtDigGenFileName;
		CDlgCheck m_chkDigGenRepeat;
		
		CDlgButton m_btnAcqNew;
		CDlgButton m_btnAcqBrowse;
		
		CDlgEdit m_edtAcqBufSize;
		CDlgEdit m_edtAcqRate;
		CDlgEdit m_edtAcqSampPerPix;
		CDlgEdit m_edtAcqFileName;
		
		CDlgCheck m_chkApplyGain;
		CDlgCombo m_cmbGainTable;
		CDlgCombo m_cmbGainSource;
		CDlgCombo m_cmbGainDest;	

		CDlgCheck m_chkRecordDigout;	
		
		CDlgCheck m_chkApplyFeedback;
		CDlgCombo m_cmbFeedback;
		CDlgCombo m_cmbFeedbackSource;
		CDlgCombo m_cmbFeedbackDest;	

		//-- Initialize controls - fill hdw list and set controls to empty values or passed setup
		void InitializeControls( ) { 
			char buf[32];

			//-- set control handles
			int iCtl;
			m_cmbHdwConfig.SetHandle( GetDlgItem( m_hDlg, IDC_CMBCONFIGNAME ), "Select device configuration to use in this session");

			m_edtGenBufSize.SetHandle( GetDlgItem( m_hDlg, IDC_EDTGENBUFSIZE ), "Size (# of samples) of each channels hardware buffer" );
			m_edtGenRate.SetHandle( GetDlgItem( m_hDlg, IDC_EDTGENRATE ), "Samples per second generated per channel" );
			for( iCtl=0; iCtl<SESSION_MAXGENCHN; ++iCtl ) { 
				m_alblGenChnName[iCtl].SetHandle( GetDlgItem( m_hDlg, IDC_LBLCH0NAME+iCtl ) );
				m_abtnGenBrowse[iCtl].SetHandle( GetDlgItem( m_hDlg, IDC_BTNGEN0BROWSE+iCtl ), "Select file" );
				m_aedtGenFileName[iCtl].SetHandle( GetDlgItem( m_hDlg, IDC_EDTCH0FILE+iCtl ), "Protocol file to generate" );
				m_achkGenRepeat[iCtl].SetHandle( GetDlgItem( m_hDlg, IDC_CHKGEN0REPEAT+iCtl ), "Whether to repeat (loop) output during synchronous acq/gen" );
				}
			
			m_btnDigGenBrowse.SetHandle( GetDlgItem( m_hDlg, IDC_BTNDIGGENBROWSE ), "Select digital generation file" );
			m_edtDigGenFileName.SetHandle( GetDlgItem( m_hDlg, IDC_EDTDIGGENFILE ), "Digital Protocol file to generate" );
			m_chkDigGenRepeat.SetHandle( GetDlgItem( m_hDlg, IDC_CHKDIGGENREPEAT ), "Whether to repeat (loop) output during synchronous acq/gen" );
			
			m_edtAcqBufSize.SetHandle( GetDlgItem( m_hDlg, IDC_EDTACQBUFSIZE ), "Size (# of samples) of each channels hardware buffer" );
			m_edtAcqRate.SetHandle( GetDlgItem( m_hDlg, IDC_EDTACQRATE ), "Samples per second acquired per channel.  Ignored for synchronous I/O" );
			m_edtAcqSampPerPix.SetHandle( GetDlgItem( m_hDlg, IDC_EDTACQSAMPPERPIX ), "Initial samples per display pixel in the scope" );
			m_edtAcqFileName.SetHandle( GetDlgItem( m_hDlg, IDC_EDTACQFILE ), "File to save acquired data during recording" );
			m_btnAcqNew.SetHandle( GetDlgItem( m_hDlg, IDC_BTNACQFILENEW ), "Make a new file name from the date and time now." );
			m_btnAcqBrowse.SetHandle( GetDlgItem( m_hDlg, IDC_BTNACQFILEBROWSE), "Select file to overwrite" );
			
			m_chkApplyGain.SetHandle( GetDlgItem( m_hDlg, IDC_CHKAPPLYGAIN), "Toggle application of gain multiplier" );
			m_cmbGainTable.SetHandle( GetDlgItem( m_hDlg, IDC_CMBGAINTABLE), "Table to lookup gain multipliers" );
			m_cmbGainSource.SetHandle( GetDlgItem( m_hDlg, IDC_CMBGAINSOURCE), "Channel which is source for gain multiplier" );
			m_cmbGainDest.SetHandle( GetDlgItem( m_hDlg, IDC_CMBGAINDEST), "Channel to which gain multiplier is applied" );

			m_chkRecordDigout.SetHandle( GetDlgItem( m_hDlg, IDC_CHKRECORDDIGOUT), "Add a simulated acquisition channel to record digital output settings." );
			
			m_chkApplyFeedback.SetHandle( GetDlgItem( m_hDlg, IDC_CHKAPPLYFEEDBACK), "Toggle application of a feedback loop" );
			m_cmbFeedback.SetHandle( GetDlgItem( m_hDlg, IDC_CMBFEEDBACK), "Selection of initial feedback configuration" );
			m_cmbFeedbackSource.SetHandle( GetDlgItem( m_hDlg, IDC_CMBFEEDBACKSOURCE), "Acquisition channel to compare to setpoint" );
			m_cmbFeedbackDest.SetHandle( GetDlgItem( m_hDlg, IDC_CMBFEEDBACKDEST), "Generation channel to calculate. (This channels 'protocol' becomes the setpoint.)" );

			//-- fill list of Hardware 
			m_oCfgs.LoadFile();
			m_cmbHdwConfig.ClearContents();
			for( int i=0; i<m_oCfgs.m_aConfigs.Count(); ++i )
				m_cmbHdwConfig.AddString( m_oCfgs.Config(i)->m_cConfigName.Buffer() );

			//-- Fill list of gain tables
			m_cmbGainTable.ClearContents();
			for( int iGT=0; iGT<m_oCfgs.m_aGainTables.Count(); ++iGT )
				m_cmbGainTable.AddString( m_oCfgs.GainTable( iGT )->m_cTableName.ConstBuffer() );
				
			//-- Fill gain source / dest lists.
			m_cmbGainSource.ClearContents();
			m_cmbGainDest.ClearContents();
			for( int iAI=0; iAI<SESSION_MAXACQCHN; ++iAI ) {
				sprintf( buf, "%d", iAI );
				m_cmbGainSource.AddString(buf);
				m_cmbGainDest.AddString(buf);
				}
						
			//-- Fill list of Feedback configurations
			m_cmbFeedback.ClearContents();
			for( int iFC=0; iFC<m_oCfgs.m_aFeedbackConfigs.Count(); ++iFC )
				m_cmbFeedback.AddString( m_oCfgs.FeedbackConfig( iFC )->m_cName.ConstBuffer() );
				
			//-- Fill feedback source / dest lists.
			m_cmbFeedbackSource.ClearContents();
			m_cmbFeedbackDest.ClearContents();
			for( int iAI=0; iAI<SESSION_MAXACQCHN; ++iAI ) {
				sprintf( buf, "%d", iAI );
				m_cmbFeedbackSource.AddString(buf);
				}
			for( int iAO=0; iAO<SESSION_MAXGENCHN; ++iAO ) {
				sprintf( buf, "%d", iAO );
				m_cmbFeedbackDest.AddString(buf);
				}

			//-- Limit text lengths 
			for( iCtl=0; iCtl<SESSION_MAXGENCHN; ++iCtl )
				m_aedtGenFileName[iCtl].SetTextLimit( _MAX_PATH );
			m_edtGenBufSize.SetTextLimit(CFGSESSION_BUFSIZEMAXLEN);		// 100mb
			m_edtGenRate.SetTextLimit(CFGSESSION_RATEMAXLEN);			// 1million 
			
			m_edtDigGenFileName.SetTextLimit( _MAX_PATH );
			
			m_edtAcqBufSize.SetTextLimit(CFGSESSION_BUFSIZEMAXLEN);		// 100mb
			m_edtAcqSampPerPix.SetTextLimit(CFGSESSION_PERPIXMAXLEN);	// 100000 samples per pixel
			m_edtAcqRate.SetTextLimit(CFGSESSION_RATEMAXLEN);			// 1million 
			m_edtAcqFileName.SetTextLimit(_MAX_PATH);
			
			m_cmbHdwConfig.SetCurSel(0);
			OnSelectConfig();
			}
			
		/*
		void EnableEdit( bool bEdit ) { 
			int iCtl;
			m_cmbHdwConfig.SetEnabled( true );

			m_edtGenBufSize.SetEnabled( bEdit );
			m_edtGenRate.SetEnabled( bEdit );
			for( iCtl=0; iCtl<SESSION_MAXGENCHN; ++iCtl ) { 
				m_aedtGenFileName[iCtl].SetEnabled( bEdit );
				m_achkGenRepeat[iCtl].SetEnabled( bEdit );
				}
			
			m_edtAcqBufSize.SetEnabled( bEdit );
			m_edtAcqRate.SetEnabled( bEdit );
			m_edtAcqSampPerPix.SetEnabled( bEdit );
			m_edtAcqFileName.SetEnabled( bEdit );
			}
		*/
			
		void Setup( CSession * pSession ) { 
			if( m_cmbHdwConfig.SetCurSel( pSession->m_cConfigName.Buffer(), false )<0 ) 
				m_cmbHdwConfig.SetCurSel(0);
			OnSelectConfig( ); 

			//-- generation
			m_edtGenBufSize.SetTextInt( pSession->m_iGenBufferSize );
			m_edtGenRate.SetTextInt( pSession->m_iGenRate );
			for( int iCtl=0; iCtl<SESSION_MAXGENCHN; ++iCtl ) { 
				m_aedtGenFileName[iCtl].SetText( pSession->m_acGenFile[iCtl].Buffer() );
				m_achkGenRepeat[iCtl].SetCheck( pSession->m_abGenRepeat[iCtl] );
				}
			m_edtDigGenFileName.SetText( pSession->m_cDigGenFile.Buffer() );
			m_chkDigGenRepeat.SetCheck( pSession->m_bDigGenRepeat );

			//-- acquisition			
			m_edtAcqBufSize.SetTextInt( pSession->m_iAcqBufferSize );
 			m_edtAcqRate.SetTextInt( pSession->m_iAcqRate );
 			m_edtAcqSampPerPix.SetTextInt( pSession->m_iAcqSampPerPixel );
			m_edtAcqFileName.SetText( pSession->m_cAcqFile.Buffer() );

			//----- advanced options
			m_chkRecordDigout.SetCheck( pSession->m_bRecordDigout );
			
			//-- gain table
			m_chkApplyGain.SetCheck( pSession->m_bGainTable );
			m_cmbGainTable.SetCurSel( pSession->m_cGainTable.ConstBuffer() );
			char buf[32];
			sprintf( buf, "%d", pSession->m_iGainTableSource );
			m_cmbGainSource.SetCurSel( buf );
			sprintf( buf, "%d", pSession->m_iGainTableApply );
			m_cmbGainDest.SetCurSel( buf );
			
			//-- Feedback
			m_chkApplyFeedback.SetCheck( pSession->m_bFeedback );
			m_cmbFeedback.SetCurSel( pSession->m_cFeedback.ConstBuffer() );
			sprintf( buf, "%d", pSession->m_iFeedbackSource );
			m_cmbFeedbackSource.SetCurSel( buf );
			sprintf( buf, "%d", pSession->m_iFeedbackApply );
			m_cmbFeedbackDest.SetCurSel( buf );

			// auto new data name?
			if ( g_oCfgs.m_oSettings.m_bAutoNewData ) {
				OnBtnAcqNew();
				}

			if ( pSession->m_bQuick ) {
				OnBtnAcqBrowse( );
				}
			}
		
		void Teardown( CSession * pSession ) { 
			m_cmbHdwConfig.GetText( &pSession->m_cConfigName );
			pSession->m_iGenBufferSize = m_edtGenBufSize.GetTextInt();
			pSession->m_iGenRate = m_edtGenRate.GetTextInt();
			for( int iCtl=0; iCtl<SESSION_MAXGENCHN; ++iCtl ) { 
				m_aedtGenFileName[iCtl].GetText( &pSession->m_acGenFile[iCtl] );
				pSession->m_abGenRepeat[iCtl] = m_achkGenRepeat[iCtl].GetCheck();
				}
			m_edtDigGenFileName.GetText( &pSession->m_cDigGenFile );
			pSession->m_bDigGenRepeat = m_chkDigGenRepeat.GetCheck();
			
			pSession->m_iAcqBufferSize = m_edtAcqBufSize.GetTextInt();
 			pSession->m_iAcqRate = m_edtAcqRate.GetTextInt();
			pSession->m_iAcqSampPerPixel = m_edtAcqSampPerPix.GetTextInt();
 			
			m_edtAcqFileName.GetText( &pSession->m_cAcqFile );
			if( pSession->m_cAcqFile.Find('.') < 0 )
				pSession->m_cAcqFile.Append(".qdf");

			//----- advanced options
			pSession->m_bRecordDigout = m_chkRecordDigout.GetCheck( );

			//-- gain table
			char buf[32];
			pSession->m_bGainTable = m_chkApplyGain.GetCheck();
			m_cmbGainTable.GetText(&pSession->m_cGainTable);
			m_cmbGainSource.GetText( buf, 31 );
			pSession->m_iGainTableSource = atoi( buf );
			m_cmbGainDest.GetText( buf, 31 );
			pSession->m_iGainTableApply = atoi( buf );

			//-- Feedback
			pSession->m_bFeedback = m_chkApplyFeedback.GetCheck();
			m_cmbFeedback.GetText(&pSession->m_cFeedback);
			m_cmbFeedbackSource.GetText( buf, 31 );
			pSession->m_iFeedbackSource = atoi( buf );
			m_cmbFeedbackDest.GetText( buf, 31 );
			pSession->m_iFeedbackApply = atoi( buf );

			}

		//-----------------------------------------------------------------------------
		// Events and messages 

		//-- [HDW CONFIG] combo selection change 
		void OnSelectConfig( ) { 
			bool bActive;
			//-- set ptr to selected config
			int iSel = m_cmbHdwConfig.GetCurSel();
			if( iSel<0 || iSel>=m_oCfgs.m_aConfigs.Count() ) 
				return;
			CConfiguration * pCfg = m_oCfgs.Config(iSel);
			
			//-- enable / disable gen controls
			// todo - disable entire if device does not support hardware 
			// Hide if < 4 
			for( int iCtl=0; iCtl<SESSION_MAXGENCHN; ++iCtl ) {
				bActive = pCfg->m_aoOutChnCfg[iCtl].m_iActive!=0;
				m_alblGenChnName[iCtl].SetText( pCfg->m_aoOutChnCfg[iCtl].m_cChannelName.Buffer() );
				m_alblGenChnName[iCtl].SetEnabled( bActive );
				m_aedtGenFileName[iCtl].SetEnabled( bActive );
				m_achkGenRepeat[iCtl].SetEnabled( bActive );
				m_abtnGenBrowse[iCtl].SetEnabled( bActive );
				}
			//-- todo .... limit rate based on hardware ?? 
			
			//-- enable/disable digital generation controls
			bActive=pCfg->HasProtocolDigGen();
			m_btnDigGenBrowse.SetEnabled(bActive);
			m_edtDigGenFileName.SetEnabled( bActive );
			m_chkDigGenRepeat.SetEnabled( bActive );
			}

		//-- [OK]
		void OnBtnOK() { 
			Teardown( m_pSession );

			//-- resolve config ptr
			int iSel = m_cmbHdwConfig.GetCurSel();
			if( iSel<0 || iSel>=m_oCfgs.m_aConfigs.Count() ) 
				return;
			CConfiguration * pCfg = m_oCfgs.Config(iSel);
			
			//-- Callback executes only when acq rate / 10 samples are ready.  Buffer should be at least 2x this.
			if( pCfg->ActiveAcqChannels() > 0 ) { 
				if( m_pSession->m_iAcqBufferSize * 5 < m_pSession->m_iAcqRate ) { 
					MessageBox( m_hDlg, "The acquisition buffer must be at least 0.2 seconds", "Session Configuration", MB_OK );
					return;
					}
				if( FileExists( m_pSession->m_cAcqFile.Buffer() ) ) 
					if( IDOK != MessageBox( m_hDlg, "The acquisition file exists already and will be overwritten.", 
									m_pSession->m_cAcqFile.Buffer(), MB_OKCANCEL | MB_ICONWARNING ) ) 
						return;
				}

			//-- Simultaneous I/O uses the same sample clock (unless feedback).  Require them to match sampling.
			if( pCfg->ActiveAcqChannels() && pCfg->ActiveGenChannels() ) { 
				if( m_pSession->m_iGenRate != m_pSession->m_iAcqRate ) { 
					MessageBox( m_hDlg, "Generation and acquisition must have the same sampling Rate.", "Session Configuration", MB_OK );
					return;
					}
				}

			//-- ensure all required analog protocols have been specified and exist 
			CString cFileName;
			for( int iCtl=0; iCtl<SESSION_MAXGENCHN; ++iCtl ) {
				if( pCfg->m_aoOutChnCfg[iCtl].m_iActive!=0 ) { 
					m_aedtGenFileName[iCtl].GetText( &cFileName );

					if( ! FileExists( cFileName.Find(".qdp")>0 
								? cFileName.Left(cFileName.Find(',')).Buffer() 
								: cFileName.Buffer() ) ) { 
						char bufmsg[100];
						sprintf(bufmsg, "Generation channel %d file does not exist", iCtl );
						MessageBox( m_hDlg, bufmsg, "QUBIO session configuration", MB_OK );
						return;
						}
					}
				}
				
			//-- ensure digital protocol exists if it is required
			if( pCfg->HasProtocolDigGen() ) { 
				m_edtDigGenFileName.GetText( &cFileName );
				if( ! FileExists( cFileName.Buffer() ) ) { 
					MessageBox( m_hDlg, "Digital generation file does not exist", "QUBIO session configuration", MB_OK );
					return;
					}
				}
			
			//-- accept changes 
			EndDialog( m_hDlg, IDOK );
			}
			
		//-- [Cancel]
		void OnBtnCancel() { 
			EndDialog( m_hDlg, IDCANCEL );
			}
						
		//-- [...] Gen file browse 
		void OnBtnGenBrowse( int iCh ) { 
			char bufName[_MAX_PATH];
			m_aedtGenFileName[iCh].GetText(bufName,_MAX_PATH);

			if( ! GetOpenFileName( 
							bufName, 
							"Protocol Files (*.qpr)\0*.qpr\0Digital protocol files (*.qdp)\0*.qdp\0All Files (*.*)\0*.*\0\0",
							"qpr", 
							GetParent(m_hDlg), NULL,
							& g_oCfgs.m_oSettings.m_cProtoPath ))
				return;
			m_aedtGenFileName[iCh].SetText( bufName );
			}
			
		//-- Digital generation file browse
		void OnBtnDigGenBrowse( ) { 
			char bufName[_MAX_PATH];
			m_edtDigGenFileName.GetText(bufName,_MAX_PATH);

			if( ! GetOpenFileName( 
							bufName, 
							"Digital protocol files (*.qdp)\0*.qdp\0All Files (*.*)\0*.*\0\0",
							"qdp", 
							GetParent(m_hDlg), NULL,
							& g_oCfgs.m_oSettings.m_cDigiProtoPath ))
				return;
			m_edtDigGenFileName.SetText( bufName );
			}
			
		//-- [...] Acq file set 
		void OnBtnAcqNew() {
			SYSTEMTIME st;
			GetLocalTime( &st );
			int(st.wYear); 
			int(st.wMonth);
			int(st.wDay) ;
			int(st.wHour);
			int(st.wMinute); 
			int(st.wSecond);

			struct tm tmNow;
			tmNow.tm_sec	= st.wSecond;
			tmNow.tm_min	= st.wMinute; 
			tmNow.tm_hour	= st.wHour;
			tmNow.tm_mday	= st.wDay ;
			tmNow.tm_mon	= st.wMonth-1;
			tmNow.tm_year	= st.wYear-1900; 

			int path_size = 0;
			char old_path[_MAX_PATH];
			m_edtAcqFileName.GetText(old_path, _MAX_PATH);
			char *old_path_sep = strrchr(old_path, '\\');
			if ( old_path_sep ) {
				path_size = (old_path_sep - old_path + 1); // include separator
			} else {
				strncpy(old_path, g_oCfgs.m_oSettings.m_cDataPath.ConstBuffer(), _MAX_PATH-2);
				path_size = strlen(old_path);
				old_path[path_size++] = '\\';
				old_path[path_size] = 0;
			}

			char buffer[_MAX_PATH];
			strncpy(buffer, old_path, path_size);
			if( strftime( buffer+path_size, _MAX_PATH-path_size, g_oCfgs.m_oSettings.m_cAcqFileName.ConstBuffer(), &tmNow ) > 0 ) 
				m_edtAcqFileName.SetText( buffer );
			else 
				m_edtAcqFileName.SetText("Data.qdf");

		}

		void OnBtnAcqBrowse( ) { 
			char bufName[_MAX_PATH];
			m_edtAcqFileName.GetText(bufName,_MAX_PATH);
			if( ! GetSaveFileName( 
							bufName, 
							"Data file (*.qdf)\0*.qdf\0All Files (*.*)\0*.*\0\0",
							"qdf", 
							GetParent(m_hDlg), NULL,
							& g_oCfgs.m_oSettings.m_cDataPath ))
				return;
			m_edtAcqFileName.SetText( bufName );
			}
					
	};

CVwConfigSession * g_pVwConfig;		// Modal - one and only instance
	
//-----------------------------------------------------------------------------
// Message Pump
BOOL CALLBACK ConfigSessionDlgProc( HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam) { 
	int wmId, wmEvent;
	
	switch( iMsg ) { 
		//----- DIALOG INIT -  Control Initialization
		case WM_INITDIALOG :		
			g_pVwConfig = new CVwConfigSession;
			g_pVwConfig->m_hDlg=hDlg;			
			g_pVwConfig->InitializeControls();
			g_pVwConfig->Setup( g_pVwConfig->m_pSession=(CSession *)lParam );

			return TRUE;

		//----- MENU and CONTROL Commands
		case WM_COMMAND : 
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			switch( wmId ) { 
				case IDC_BTNGEN0BROWSE : 
				case IDC_BTNGEN1BROWSE : 
				case IDC_BTNGEN2BROWSE : 
				case IDC_BTNGEN3BROWSE :
					g_pVwConfig->OnBtnGenBrowse( wmId-IDC_BTNGEN0BROWSE );
					break;

				case IDC_BTNDIGGENBROWSE :
					g_pVwConfig->OnBtnDigGenBrowse( );
					break;

				case IDC_BTNACQFILENEW:
					g_pVwConfig->OnBtnAcqNew();
					break;

				case IDC_BTNACQFILEBROWSE :
					g_pVwConfig->OnBtnAcqBrowse( );
					break;
					
				case IDC_CMBCONFIGNAME :			//-- {Config} Combo Box
					if( wmEvent==CBN_SELCHANGE ) 
						g_pVwConfig->OnSelectConfig( ); 
					break;

				case IDCANCEL :						//-- [Cancel]
					g_pVwConfig->OnBtnCancel();
					break;
					
				case IDOK :							//-- [OK]
					g_pVwConfig->OnBtnOK();
					break;
				}
			return TRUE;

		case WM_PAINT :
			/*		
			PAINTSTRUCT ps; 
			HDC hdc = BeginPaint(hDlg, &ps); 
			...extra drawing... 
			EndPaint(hDlg, &ps); 
			*/
			return false;

		case WM_DESTROY : 
			delete g_pVwConfig;
			SetWindowLong( GetParent( hDlg ), 0, 0);
			return TRUE;
			
		default : 
			break;
		}
	return FALSE;
	} 

